#!/bin/sh

# PEM to PFX
pem2pfx() {
    local FULLNAME=$1.pem
    local KEYNAME=$2.key
    local CANAME=$3.pem
    local RESFILE=$4.pfx

    [ -f $FULLNAME ] || FULLNAME=$1.crt
    [ -f $FULLNAME ] || FULLNAME=$1.cer
    [ -f $FULLNAME ] || {
	echo "No certificate files $1.pem, $1.crt or $1.cer found. Exiting with error"
	exit 1
    }
    [ -f $KEYNAME ] || {
	echo "No key file $KEYNAME found. Exiting with error"
	exit 1
    }
    [ -f $CANAME ] || CANAME=$3.crt
    [ -f $CANAME ] || CANAME=$3.cer
    [ -f $CANAME ] || {
	echo "No CA certificate files $3.pem, $3.crt or $3.cer found. Exiting with error"
	exit 1
    }
    openssl pkcs12 -export -passout pass: -out $RESFILE -inkey $KEYNAME -in $FULLNAME -certfile $CANAME
    return $?
}

# PFX to PEM
pfx2pem() {
    local FULLNAME=$1.pfx

    [ -f $FULLNAME ] || FULLNAME=$1.p12
    [ -f $FULLNAME ] || {
	echo "No certificate files $1.pfx or $1.p12 found. Exiting with error"
	exit 1
    }
    openssl pkcs12 -in $FULLNAME -out $1-key.pem -nodes -nocerts -passin pass: -passout pass:
    [ $? -eq 0 ] && {
	openssl pkcs12 -in $FULLNAME -out $1.pem -nodes -clcerts -nokeys -passin pass: -passout pass:
	[ $? -eq 0 ] && openssl pkcs12 -in $FULLNAME -out $1-ca.pem -nodes -cacerts -nokeys -passin pass: -passout pass:
    }
    return $?
}

# DER to PEM
der2pem() {
    local FULLNAME=$1.der

    openssl x509 -inform der -in $FULLNAME -out $1.pem
    return $?
}

# PEM to DER
pem2der() {
    local FULLNAME=$1.pem

    [ -f $FULLNAME ] || {
	echo "No certificate file $1.pem at input. Exiting with error"
	exit 1
    }
    openssl x509 -outform der -in $FULLNAME -out $1.der
    return $?
}

# PEM to PKCS7 (P7B)
pem2p7b() {
    local FULLNAME=$1.pem
    local CANAME=$2.pem
    local RESFILE=$3.p7b

    [ -f $FULLNAME ] || FULLNAME=$1.crt
    [ -f $FULLNAME ] || FULLNAME=$1.cer
    [ -f $FULLNAME ] || {
	echo "No certificate files $1.pem, $1.crt or $1.cer found. Exiting with error"
	exit 1
    }
    [ -f $CANAME ] || CANAME=$2.crt
    [ -f $CANAME ] || CANAME=$2.cer
    [ -f $CANAME ] || {
	echo "No CA certificate files $2.pem, $2.crt or $2.cer found. Exiting with error"
	exit 1
    }
    openssl crl2pkcs7 -nocrl -certfile $FULLNAME -out $RESFILE -certfile $CANAME
    return $?
}

# PKCS7 (P7B) to PEM
p7b2pem() {
    local FULLNAME=$1.p7b

    openssl pkcs7 -print_certs -in $FULLNAME -out $1.pem
    return $?
}

[ -z "$2" ] && {
	echo "Usage: $0 CONVERTION CERTIFACTE_NAME [...]

Supported convertion types are:
    pem2der
    pem2pfx
    pem2p7b
    der2pem
    pfx2pem
    p7b2pem

    CERTIFICATE_NAME must be specified without extension"
	exit 1
}

case "$1" in
    pem2pfx)
	[ -z "$5" ] && {
	    echo "pem2pfx comvertion requires 3 additional params: KEY_NAME CA_NAME RESULT_NAME"
	    exit 1
	}
	shift
	pem2pfx $*
    ;;
    pem2der)
	shift
	pem2der $*
    ;;
    pem2p7b)
	[ -z "$4" ] && {
	    echo "pem2p7b comvertion requires 2 additional params: CA_NAME RESULT_NAME"
	    exit 1
	}
	shift
	pem2p7b $*
    ;;
    der2pem)
	shift
	der2pem $*
    ;;
    pfx2pem)
	shift
	pfx2pem $*
    ;;
    p7b2pem)
	shift
	p7b2pem $*
    ;;
    *)
	echo "Unsupported convertion type.

Supported convertion types are:
    pem2der
    pem2pfx
    pem2p7b
    der2pem
    pfx2pem
    p7b2pem
"
	exit 1
    ;;
esac

exit $?
